@@ -1,10 +1,13 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
+from __future__ import annotations
+
 import collections
 import copy
 import itertools
 import math
+from typing import List, Optional
 
 from .elements import elements_dict
 from .exceptions import *
@@ -54,24 +57,88 @@ REACTOR_TYPES = {
                                'quantum-walls-y': {'5': [0, 1, 2, 3, 4, 5, 6, 7]}}}
 
 
-class Pipe(list):
+class Pipe:
     """A SpaceChem component's pipe. All posns are relative to the parent component's posn."""
-    __slots__ = 'posns',
+    __slots__ = 'posns', '_molecules', '_add_cycles', '_last_pop_cycle'
 
-    def __init__(self, posns):
+    def __init__(self, posns: List[Position]):
         """Construct a pipe. posns should be defined relative to the pipe's parent component posn."""
-        super().__init__([None for _ in posns])
         self.posns = posns
+        # To avoid incurring any performance costs for moving molecules through the pipe, implement a pipe as a timed
+        # queue, storing the cycle each molecule was added on and only allowing them to exit the pipe if their age in
+        # cycles is >= len(self) - 1 (e.g. for a 1-long pipe, they are available in the same cycle).
+        self._molecules = collections.deque()
+        self._add_cycles = collections.deque()  # Kept in lockstep with _molecules
+        # To ensure reactors can't input from a pipe twice in a cycle, track the last cycle of a successful pop()
+        self._last_pop_cycle = -1
 
-    def move_contents(self):
-        """Shift all molecules in this pipe forward one if possible."""
-        # Iterate from the back to make clog-handling simple
-        for i in range(len(self) - 2, -1, -1):  # Note that we don't need to shift the last element
-            if self[i + 1] is None:
-                self[i], self[i + 1] = None, self[i]
+    def __len__(self):
+        return len(self.posns)
+
+    def get(self, idx: int, cycle: int) -> Optional[Molecule]:
+        """Return the molecule at the given index, else None."""
+        if not self._molecules:
+            return None
+        # Provide O(1) access for either end which is all we use, else fall back to O(N)
+        elif idx == 0:
+            if (len(self._molecules) == len(self)  # Pipe is full
+                    or cycle == self._add_cycles[0]):  # A molecule was already added this cycle
+                return self._molecules[0]
+            else:
+                return None
+        elif idx == -1:
+            # Make sure it's had time to reach the end
+            if cycle - self._add_cycles[-1] >= len(self) - 1 and cycle != self._last_pop_cycle:
+                return self._molecules[-1]
+            else:
+                return None
+        else:
+            return self.to_list(cycle)[idx]
+
+    def push(self, molecule: Molecule, cycle: int) -> bool:
+        """Attempt to pass the given molecule to the pipe. Return False if there is no room at the front of the pipe.
+        Note that since cycle incrementation controls pipe movement, if a molecule is being added that shouldn't be
+        moved in the same cycle (namely, all components except reactor), cycle + 1 should be given as the cycle the
+        molecule was added instead.
+        """
+        if self._molecules and (len(self._molecules) == len(self)  # Pipe is full
+                                or cycle == self._add_cycles[0]):  # A molecule was already added this cycle
+            return False
+        else:
+            self._molecules.appendleft(molecule)
+            self._add_cycles.appendleft(cycle)
+            return True
+
+    def pop(self, cycle: int) -> Optional[Molecule]:
+        """Remove and return the molecule at the end of the pipe, or None if there is none."""
+        if (not self._molecules
+                or cycle - self._add_cycles[-1] < len(self) - 1  # Make sure it's had time to reach the end
+                or cycle == self._last_pop_cycle):  # Make sure a component can't extract two molecules in a cycle
+            return None
+        else:
+            self._last_pop_cycle = cycle
+            self._add_cycles.pop()
+            return self._molecules.pop()
+
+    def to_list(self, cycle: int) -> list:
+        """Return a list representing the current positions of molecule in the pipe, with None for empty spaces."""
+        result = [None for _ in self.posns]
+
+        # Insert molecules starting from the back, to account for backed up molecules
+        cur_pipe_idx = len(self) - 1
+        for molecule, cycle_added in zip(reversed(self._molecules), reversed(self._add_cycles)):
+            # Add this molecule in the farthest position it could have travelled to, accounting for clogs
+            if cycle - cycle_added >= cur_pipe_idx:
+                result[cur_pipe_idx] = molecule
+                # Update the current pipe idx
+                cur_pipe_idx -= 1
+            else:  # Once we are past any clogged molecules, we will never need cur_pipe_idx again
+                result[cycle - cycle_added] = molecule
+
+        return result
 
     @classmethod
-    def from_preset_string(cls, start_posn, dirns_str):
+    def from_preset_string(cls, start_posn: Position, dirns_str: str) -> Pipe:
         """Construct a pipe from the given CE pipe string, e.g. 'RRDRUULR', moving in the indicated directions
         (U = Up, R = Right, D = Down, L = Left) from the start_posn (should be relative to the parent component's posn).
         """
@@ -85,7 +152,7 @@ class Pipe(list):
         return Pipe(posns)
 
     @classmethod
-    def from_export_str(cls, export_str):
+    def from_export_str(cls, export_str: str):
         """Note that a pipe's solution lines might not be contiguous. It is expected that the caller filters
         out the lines for a single pipe and passes them as a single string to this method.
         """
@@ -110,10 +177,16 @@ class Pipe(list):
 
         return Pipe(posns)
 
-    def export_str(self, pipe_idx=0):
+    def export_str(self, pipe_idx: int = 0) -> str:
         """Represent this pipe in solution export string format."""
         return '\n'.join(f'PIPE:{pipe_idx},{posn.col},{posn.row}' for posn in self.posns)
 
+    def reset(self):
+        """Empty this pipe."""
+        self._molecules = collections.deque()
+        self._add_cycles = collections.deque()
+        self._last_pop_cycle = -1
+
 
 class Component:
     """Informal Interface class defining methods overworld objects will implement one or more of."""
@@ -151,6 +224,8 @@ class Component:
             return super().__new__(TeleporterInput)
         elif _type == 'drag-qpipe-out':
             return super().__new__(TeleporterOutput)
+        elif 'weapon' in parts:
+            return super().__new__(Weapon)
         else:
             raise ValueError(f"Unrecognized component type {_type}")
 
@@ -236,15 +311,13 @@ class Component:
         return
 
     def move_contents(self, _):
-        """Move the contents of this object (e.g. waldos/molecules), including its pipes."""
-        for pipe in self.out_pipes:
-            pipe.move_contents()
+        """Move the contents of this object (e.g. waldos/molecules)."""
+        pass
 
     def reset(self):
         """Reset this component and its pipe's contents as if it has never been run."""
         for pipe in self.out_pipes:
-            for i in range(len(pipe)):
-                pipe[i] = None
+            pipe.reset()
 
         return self
 
@@ -306,12 +379,10 @@ class Input(Component):
         self.num_inputs = 0
 
     def move_contents(self, cycle):
-        """Move pipe contents, then add a new molecule to pipe if on correct cycle and there is room."""
-        super().move_contents(cycle)
-
+        """Create a new molecule if on the correct cycle and the pipe has room."""
         # -1 necessary since starting cycle is 1 not 0, while mod == 1 would break on rate = 1
-        if (cycle - 1) % self.input_rate == 0 and self.out_pipe[0] is None:
-            self.out_pipe[0] = copy.deepcopy(self.molecules[0])
+        if (cycle - 1) % self.input_rate == 0 and self.out_pipe.get(0, cycle) is None:
+            self.out_pipe.push(copy.deepcopy(self.molecules[0]), cycle + 1)
             self.num_inputs += 1
 
     def reset(self):
@@ -355,12 +426,9 @@ class RandomInput(Input):
         return self.random_bucket.pop(bucket_idx)
 
     def move_contents(self, cycle):
-        """Move pipe contents, then add a new molecule to pipe if on correct cycle and there is room."""
-        super(Input, self).move_contents(cycle)
-
         # -1 necessary since starting cycle is 1 not 0, while mod == 1 would break on rate = 1
-        if (cycle - 1) % self.input_rate == 0 and self.out_pipe[0] is None:
-            self.out_pipe[0] = copy.deepcopy(self.molecules[self.get_next_molecule_idx()])
+        if (cycle - 1) % self.input_rate == 0 and self.out_pipe.get(0, cycle) is None:
+            self.out_pipe.push(copy.deepcopy(self.molecules[self.get_next_molecule_idx()]), cycle + 1)
             self.num_inputs += 1
 
     def reset(self):
@@ -393,16 +461,13 @@ class ProgrammedInput(Input):
         self.num_inputs = 0
 
     def move_contents(self, cycle):
-        """Move pipe contents, then add a new molecule to pipe if on correct cycle and there is room."""
-        super(Input, self).move_contents(cycle)
-
         # -1 necessary since starting cycle is 1 not 0, while mod == 1 would break on rate = 1
-        if (cycle - 1) % self.input_rate == 0 and self.out_pipe[0] is None:
+        if (cycle - 1) % self.input_rate == 0 and self.out_pipe.get(0, cycle) is None:
             if self.starting_idx == len(self.starting_molecules):
-                self.out_pipe[0] = copy.deepcopy(self.repeating_molecules[self.repeating_idx])
+                self.out_pipe.push(copy.deepcopy(self.repeating_molecules[self.repeating_idx]), cycle + 1)
                 self.repeating_idx = (self.repeating_idx + 1) % len(self.repeating_molecules)
             else:
-                self.out_pipe[0] = copy.deepcopy(self.starting_molecules[self.starting_idx])
+                self.out_pipe.push(copy.deepcopy(self.starting_molecules[self.starting_idx]), cycle + 1)
                 self.starting_idx += 1
 
             self.num_inputs += 1
@@ -438,20 +503,18 @@ class Output(Component):
         self.target_count = output_dict['count']
         self.current_count = 0
 
-    def do_instant_actions(self, _):
+    def do_instant_actions(self, cycle):
         """Check for and process any incoming molecule, and return True if this output just completed (in which case
         the caller should check if the other outputs are also done). This avoids checking all output counts every cycle.
         """
         if self.in_pipe is None:
             return False
 
-        molecule = self.in_pipe[-1]
+        molecule = self.in_pipe.pop(cycle)
         if molecule is not None:
             if not molecule.isomorphic(self.output_molecule):
                 raise InvalidOutputError(f"Invalid output molecule; expected:\n{self.output_molecule}\n\nbut got:\n{molecule}")
 
-            self.in_pipe[-1] = None
-
             if self.current_count < self.target_count:
                 self.current_count += 1
 
@@ -494,23 +557,19 @@ class PassThroughCounter(Output):
         if self.in_pipe is None:
             return False
 
-        # TODO: I thought from eyeballing it that a molecule should go in and out of a pass-through counter
-        #       within the same cycle, but it seems I had to put this block first to avoid an off-by-1-cycle error.
-        #       Double-check what's going on here, I think this may diverge from SC when the pass-through's pipe gets
-        #       clogged
-        #       Probably because it can't be teleported in the same cycle or it will be moved that cycle too, which
-        #       effectively makes it move twice
-        # If there is a molecule stored (possibly stored just now), put it in the output pipe if possible
-        if self.stored_molecule is not None and self.out_pipe[0] is None:
-            self.stored_molecule, self.out_pipe[0] = None, self.stored_molecule
-
         # If the stored slot is empty, store the next molecule and 'output' it while we do so
-        if self.in_pipe[-1] is not None and self.stored_molecule is None:
-            self.stored_molecule = self.in_pipe[-1]
-            return super().do_instant_actions(cycle)  # This will set self.in_pipe[-1] to None
+        if self.in_pipe.get(-1, cycle) is not None and self.stored_molecule is None:
+            self.stored_molecule = self.in_pipe.get(-1, cycle)
+            return super().do_instant_actions(cycle)  # This will remove the molecule from the pipe
 
         return False
 
+    def move_contents(self, cycle):
+        # If there is a molecule stored (possibly stored just now), put it in the output pipe if possible
+        if self.stored_molecule is not None and self.out_pipe.get(0, cycle) is None:
+            self.out_pipe.push(self.stored_molecule, cycle + 1)
+            self.stored_molecule = None
+
     def reset(self):
         super().reset()
         self.stored_molecule = None
@@ -531,10 +590,10 @@ class DisabledOutput(Component):
     def __init__(self, *, _type, posn):
         super().__init__(_type=_type, posn=posn, num_in_pipes=1)
 
-    def do_instant_actions(self, _):
+    def do_instant_actions(self, cycle):
         # Technically should check for `in_pipe is None` first but I'd also be curious to see this crash since disabled
         # outputs are only used in research levels, where it should be impossible to not connect to the disabled output
-        if self.in_pipe[-1] is not None:
+        if self.in_pipe.get(-1, cycle) is not None:
             raise InvalidOutputError("A molecule was passed to a disabled output.")
 
 
@@ -554,11 +613,11 @@ class OutputPrinter(Component):
     def in_pipe(self, p):
         self.in_pipes[0] = p
 
-    def do_instant_actions(self, _):
+    def do_instant_actions(self, cycle):
         """Consume and print incoming molecules."""
         if self.in_pipe is not None:
             # TODO: Print received molecules when in --debug somehow
-            self.in_pipe[-1] = None
+            self.in_pipe.pop(cycle)
 
 
 class PassThroughPrinter(OutputPrinter):
@@ -586,12 +645,13 @@ class PassThroughPrinter(OutputPrinter):
             return
 
         # If there is a molecule stored (possibly stored just now), put it in the output pipe if possible
-        if self.stored_molecule is not None and self.out_pipe[0] is None:
-            self.stored_molecule, self.out_pipe[0] = None, self.stored_molecule
+        if self.stored_molecule is not None and self.out_pipe.get(0, cycle) is None:
+            self.out_pipe.push(self.stored_molecule, cycle + 1)
+            self.stored_molecule = None
 
         # If the stored slot is empty, store the next molecule and 'print' it while we do so
-        if self.in_pipe[-1] is not None and self.stored_molecule is None:
-            self.stored_molecule = self.in_pipe[-1]
+        if self.in_pipe.get(-1, cycle) is not None and self.stored_molecule is None:
+            self.stored_molecule = self.in_pipe.get(-1, cycle)
             super().do_instant_actions(cycle)  # This will consume and print the input molecule
 
     def reset(self):
@@ -608,10 +668,10 @@ class Recycler(Component):
     def __init__(self, component_dict=None, _type=None, posn=None):
         super().__init__(component_dict=component_dict, _type=_type, posn=posn, num_in_pipes=3)
 
-    def do_instant_actions(self, _):
+    def do_instant_actions(self, cycle):
         for pipe in self.in_pipes:
             if pipe is not None:
-                pipe[-1] = None
+                pipe.pop(cycle)
 
 
 # TODO: Ideally this would subclass both deque and Component but doing so gives me
@@ -642,23 +702,17 @@ class StorageTank(Component):
     def out_pipe(self, p):
         self.out_pipes[0] = p
 
-    def do_instant_actions(self, _):
+    def do_instant_actions(self, cycle):
         if self.in_pipe is None:
             return
 
-        if self.in_pipe[-1] is not None and len(self.contents) < self.MAX_CAPACITY:
-            self.contents.append(self.in_pipe[-1])
-            self.in_pipe[-1] = None
+        if self.in_pipe.get(-1, cycle) is not None and len(self.contents) < self.MAX_CAPACITY:
+            self.contents.appendleft(self.in_pipe.pop(cycle))
 
     def move_contents(self, cycle):
-        """Fill the output pipe if the storage tank is not empty. This does not occur in do_instant_actions to reflect
-        the fact that a reactor-tank-reactor setup cannot pass a molecule between reactors in the same cycle, unlike
-        with two adjacent reactors.
-        """
-        super().move_contents(cycle)
-
-        if self.out_pipe[0] is None and self.contents:
-            self.out_pipe[0] = self.contents.popleft()
+        """Add a molecule to the output pipe if the storage tank is not empty."""
+        if self.out_pipe.get(0, cycle) is None and self.contents:
+            self.out_pipe.push(self.contents.pop(), cycle + 1)
 
     @classmethod
     def from_export_str(cls, export_str):
@@ -701,7 +755,7 @@ class TeleporterInput(Component):
     def in_pipe(self, p):
         self.in_pipes[0] = p
 
-    def do_instant_actions(self, _):
+    def do_instant_actions(self, cycle):
         """Note that the teleporter pair behaves differently from a pass-through counter insofar as the pass-through
         counter stores any molecule it receives internally when its output pipe is clogged, whereas the teleporter
         refuses to accept the next molecule until the output pipe is clear (i.e. behaves like a single discontinuous
@@ -710,23 +764,20 @@ class TeleporterInput(Component):
         if self.in_pipe is None:
             return
 
-        molecule = self.in_pipe[-1]
-        if molecule is not None and self.destination.out_pipe[0] is None:
-            assert len(molecule) == 1, f"An invalid molecule was passed to Teleporter (Input): {molecule}"
+        if self.destination.out_pipe.get(0, cycle) is None:
+            molecule = self.in_pipe.pop(cycle)
+            if molecule is not None:
+                assert len(molecule) == 1, f"An invalid molecule was passed to Teleporter (Input): {molecule}"
 
-            self.in_pipe[-1], self.destination.molecule = None, molecule
+                self.destination.out_pipe.push(molecule, cycle + 1)
 
 
 class TeleporterOutput(Component):
     DEFAULT_SHAPE = (3, 1)
-    # TODO: Needing an internal molecule slot is awkward but I couldn't find a cleaner way to avoid the molecule
-    #       being both teleported and moved in the same cycle if the teleporters have the wrong relative component
-    #       priorities
-    __slots__ = 'destination', 'molecule'
+    __slots__ = ()
 
     def __init__(self, component_dict):
         super().__init__(component_dict, num_out_pipes=1)
-        self.molecule = None
 
     # Convenience properties
     @property
@@ -737,18 +788,6 @@ class TeleporterOutput(Component):
     def out_pipe(self, p):
         self.out_pipes[0] = p
 
-    def move_contents(self, cycle):
-        # Move pipe contents first, then add the teleported molecule to the front to avoid double-moving it
-        super().move_contents(cycle)
-        if self.out_pipe[0] is None:
-            self.molecule, self.out_pipe[0] = None, self.molecule
-
-    def reset(self):
-        super().reset()
-        self.molecule = None
-
-        return self
-
 
 class Reactor(Component):
     DEFAULT_SHAPE = (4, 4)  # Size in overworld
@@ -1254,14 +1293,12 @@ class Reactor(Component):
 
         return result
 
-    def do_instant_actions(self, _):
+    def do_instant_actions(self, cycle):
         for waldo in self.waldos:
-            self.exec_instrs(waldo)
+            self.exec_instrs(waldo, cycle)
 
     def move_contents(self, cycle):
         """Move all waldos in this reactor and any molecules they are holding."""
-        super().move_contents(cycle)
-
         # If the waldo is facing a wall, mark it as stalled (may also be stalled due to sync, input, etc.)
         for waldo in self.waldos:
             if ((waldo.direction == UP and waldo.position.row == 0)
@@ -1420,7 +1457,7 @@ class Reactor(Component):
         self.check_wall_collisions(molecule)
         self.check_quantum_wall_collisions(molecule)
 
-    def exec_instrs(self, waldo):
+    def exec_instrs(self, waldo, cycle):
         if waldo.position not in waldo.instr_map:
             return
 
@@ -1434,9 +1471,9 @@ class Reactor(Component):
         if cmd is None:
             return
         elif cmd.type == InstructionType.INPUT:
-            self.input(waldo, cmd.target_idx)
+            self.input(waldo, cmd.target_idx, cycle)
         elif cmd.type == InstructionType.OUTPUT:
-            self.output(waldo, cmd.target_idx)
+            self.output(waldo, cmd.target_idx, cycle)
         elif cmd.type == InstructionType.GRAB:
             self.grab(waldo)
         elif cmd.type == InstructionType.DROP:
@@ -1479,17 +1516,16 @@ class Reactor(Component):
         elif cmd.type == InstructionType.PAUSE:
             raise PauseException("Pause command encountered")
 
-    def input(self, waldo, input_idx):
+    def input(self, waldo, input_idx, cycle):
         # If there is no such pipe or it has no molecule available, stall the waldo
         if (input_idx > len(self.in_pipes) - 1
                 or self.in_pipes[input_idx] is None
-                or self.in_pipes[input_idx][-1] is None):
+                or self.in_pipes[input_idx].get(-1, cycle) is None):
             waldo.is_stalled = True
             return
 
         # Grab the molecule from the appropriate pipe or stall if no such molecule (or no pipe)
-        new_molecule = self.in_pipes[input_idx][-1]
-        self.in_pipes[input_idx][-1] = None
+        new_molecule = self.in_pipes[input_idx].pop(cycle)
 
         sample_posn = next(iter(new_molecule.atom_map))
         # If the molecule came from a previous reactor, shift its columns from output to input co-ordinates
@@ -1507,7 +1543,7 @@ class Reactor(Component):
 
         self.check_molecule_collisions_lazy(new_molecule)
 
-    def output(self, waldo, output_idx):
+    def output(self, waldo, output_idx, cycle):
         # If the there is no such output pipe (e.g. assembly reactor, large output research), do nothing
         if (output_idx > len(self.out_pipes) - 1
                 or self.out_pipes[output_idx] is None):
@@ -1529,15 +1565,24 @@ class Reactor(Component):
 
         # Try to output the first molecule in the zone if an output hasn't already been done this cycle
         if molecule is not None:
-            if self.out_pipes[output_idx][0] is None:
-                # Put the molecule in the pipe
-                self.out_pipes[output_idx][0] = molecule
+            # Most components (input, storage tanks, quantum pipes, etc.) behave as though the molecule gets put into
+            # their pipe in the movement phase, with an animation of the molecule moving out of the component, and the
+            # molecule not being available to the downstream component until the next cycle's instant actions phase.
+            # Reactors behave differently, with the spawned molecule appearing directly in the next segment without
+            # movement animation, and being available immediately during the same instant actions phase for the next
+            # component to consume. To support this with the updated dependence of pipes on the cycle count, treat the
+            # molecule as though it was pushed during the previous movement phase (-1 to cycle count since instant
+            # actions come before movement in a cycle).
+            # TODO: Should figure out if there's a way to make this less awkward.
+            if self.out_pipes[output_idx].get(0, cycle) is None:
+                # Put the molecule in the pipe and remove it from the reactor
+                self.out_pipes[output_idx].push(molecule, cycle)
 
                 # Look for any other outputable molecule
-                molecule = next(molecules_in_zone, None)  # Look for another outputable molecule
+                molecule = next(molecules_in_zone, None)
 
-                # Remove the outputted molecule from the reactor (make sure not to use the iterator again now!)
-                del self.molecules[self.out_pipes[output_idx][0]]
+                # Remove the just-output molecule from the reactor (AFTER finishing with the dict iterator)
+                del self.molecules[self.out_pipes[output_idx].get(0, cycle)]
 
         # If there is any output(s) remaining in this zone (regardless of whether we outputted), stall this waldo
         waldo.is_stalled = molecule is not None
@@ -1798,3 +1843,66 @@ class Reactor(Component):
             waldo.reset()
 
         return self
+
+
+# Component used in defense levels to damage a boss
+# In order to re-use as much existing logic as possible, we can treat these like outputs, which damage the boss when
+# completed if particular conditions are met.
+# 'drag-weapon-canister'
+# 'drag-weapon-consumer'... but it's just a typed recycler lol?
+class Weapon(Output):
+    __slots__ = ()  # TODO: 'boss' and any other properties common to other defense level weapons once implemented
+
+    def __new__(cls, component_dict, _type=None, **kwargs):
+        """Convert to the specific weapon subclass based on component name."""
+        _type = component_dict['type'] if _type is None else _type
+        if _type == 'drag-weapon-canister':
+            return super().__new__(CrashCanister)
+        elif _type == 'drag-weapon-consumer':
+            return super().__new__(InternalStorageTank)
+        else:
+            raise ValueError(f"Invalid weapon type `{component_dict['type']}`")
+
+    def __init__(self, component_dict, *args, **kwargs):
+        super().__init__(output_dict=component_dict, *args, **kwargs)
+
+
+class CrashCanister(Weapon):
+    """Collapsar. While it's categorized as a weapon, it's effectively just an output with some end delay."""
+    __slots__ = 'canister_drop_cycle',
+    DEFAULT_SHAPE = (4, 4)
+
+    def __init__(self, component_dict, *args, **kwargs):
+        super().__init__(component_dict, *args, **kwargs)
+        self.canister_drop_cycle = None
+
+    # For most defense levels, the animation at the end halts all waldos, causing them to repeat the last cycle until
+    # the animation completes
+    # However in collapsar, when the output is complete it stops accepting molecules, while the canister drops, but the
+    # solution keeps running as normal and must not crash due to any clogs that result.
+    # To simulate this, instead of using an end-animation-cycles var like we'll do for other defense levels, have the
+    # canister not mark itself as complete until 2000 cycles after the 40th output
+    def do_instant_actions(self, cycle):
+        # Behave like a normal output until complete
+        if self.canister_drop_cycle is None:
+            if super().do_instant_actions(cycle):
+                # Once complete, save the cycle and artificially lower the complete count to ensure the solution can't
+                # end even if another output completes (since we're treating the hydrogen tank as a 0-count output)
+                self.current_count -= 1
+                self.canister_drop_cycle = cycle
+        elif cycle == self.canister_drop_cycle + 2000:
+            # Once the target count is met, stop processing inputs and wait for 2000 cycles before setting the target
+            # count back to its true value and indicating to the caller that the output just completed
+            self.current_count += 1
+            return True
+
+        return False
+
+    def reset(self):
+        super().reset()
+        self.canister_drop_cycle = None
+
+
+class InternalStorageTank(Weapon):
+    """Collapsar. While its component name indicates it's a weapon, it's effectively a 0-count output."""
+    pass
